<?php

/**
 * @file
 * Contains \Drupal\cforge\Forms\Settings.
 *
 */
namespace Drupal\cforge\Forms;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class Settings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormID() {
    return 'cforge_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildform($form, $form_state);
    $config = $this->config('cforge.settings');
    $form['mail'] = [
      '#title' => $this->t('Mail'),
      '#type' => 'details',
      'mail_footer' => [
        '#title' => $this->t('Footer'),
        '#description' => $this->t('Appended to all mails sent by the site to members. HTML is allowed.'),
        '#type' => 'textarea',
        '#rows' => 4,
        '#default_value' => $config->get('mail_footer')
      ],
      //the broadcast settings would go here
      '#weight' => 2
    ];

    Self::buildLoginOptions($form);
    $form['name_mode']['#weight'] = 3;

    $form['robots_protect'] = [
      '#title' => t('Conceal from search engines'),
      '#description' => t('Rewrites the robots.txt file'),
      '#type' => 'checkbox',
      '#default_value' => $config->get('robots_protect', 0),
      '#weight' => 4
    ];
    $form['referral'] = [
      '#title' => t('Referral'),
      '#description' => t('This user will receive a reward when a new account is created'),
      '#type' => 'fieldset',
      'referral_fee' => [
        '#title' => t('Referral fee'),
        '#type' => 'worths_form',
        '#default_value' => $config->get('referral_fee'),
        '#config' => TRUE
      ],
      'referral_src' => [
        '#title' => t('Referral source wallet'),
        '#description' => t('Wallet from which referal payments are drawn.'),
        '#type' => 'wallet_entity_auto',
        '#default_value' => \Drupal\mcapi\Entity\Wallet::load($config->get('referral_src')),
        '#weight' => 3
      ],
      '#weight' => 3
    ];

    $terminology = [];
    foreach ($config->get('terminology') as $key => $value) {
      $terminology[] = "$key|$value";
    }

    $form['terminology'] = [
      '#title' => $this->t('Replace words'),
      '#description' => $this->t('This is applied to all interface strings after translation. Not config or content. Case sensitive.'),
      '#type' => 'textarea',
      '#default_value' => implode("\n", $terminology),
      '#element_validate' => [[get_class(), 'validateTerminology']],
      '#rows' => 8,
    ];
    return $form;
  }

  public static function validateTerminology(&$element, $form_state) {
    $array = static::extractAllowedValues($element['#value']);
    if (!is_array($array)) {
      $form_state->setError($element, t('Could not parse terminology field'));
    }
    $form_state->setValue('terminology', $array);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    //can't have a referral fee without a wallet
    if (!empty_worths_config($form_state->getValue('referral_fee'))) {
      if (!$form_state->getValue('referral_src')) {
        $form_state->setError(
          $form['referral']['referral_src'],
          t('No wallet has been specified to pay the referral fee')
        );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $config = $this->configFactory->getEditable('cforge.settings');
    $config
      ->set('mail_footer', $values['mail_footer'])
      ->set('robots_protect', $values['robots_protect'])
      ->set('referral_fee', $form_state->getValue('referral_fee'))
      ->set('referral_src', $form_state->getValue('referral_src'))
      ->set('terminology', $form_state->getValue('terminology'))
      ->save();
  }


  function getEditableConfigNames() {
    return ['cforge.settings', 'realname.settings'];
  }

  static function buildLoginOptions(&$form, $weight = 0) {
    $form['names'] = [
      '#title' => t('Names'),
      '#description' => t('Every user has a unique login name and a full names which is comprised of first name plus last name.'),
      '#type' => 'details',
      '#open' => TRUE,
      '#weight' => $weight
    ];
    $form['names']['real_name_login'] = [
      '#title' =>  t('Allow login using full name'),
      '#description' =>  t('Full names would be validated for uniqueness'),
      '#type' => 'checkbox',
      '#default_value' => \Drupal::configFactory()->get('realname.settings')->get('login'),
      '#weight' => 1,
    ];
    $form['names']['account_name_display'] = [
      '#title' =>  t('Display the login name in brackets after the real name'),
      '#type' => 'checkbox',
      '#default_value' => \Drupal::config('cforge.settings')->get('account_name_display'),
      '#weight' => 3
    ];
    $form['names']['name_legacy_mode'] = [
      '#type' => 'value',
      '#value' => FALSE,
      '#weight' => 2
    ];

    $form['#submit'][] = ['\Drupal\cforge\Forms\Settings', 'submitLoginOptions'];
  }

  /**
   * form submit callback, used here but also in HamletsConfigureForm as a setup option
   *
   * @param type $form
   * @param type $form_state
   */
  static function submitLoginOptions($form, $form_state) {
    \Drupal::configFactory()
      ->getEditable('cforge.settings')
      ->set('account_name_display', $form_state->getValue('account_name_display'))
      ->save();
        //realname module settings @todo this should be overridden with a config file
    \Drupal::configFactory()
      ->getEditable('realname.settings')
      ->set('login', $form_state->getValue('real_name_login'))
      ->save();
  }


  /**
   * Extracts the allowed values array from the terminology element.
   *
   * @param string $string
   *   The raw string to extract values from.
   *
   * @return array|null
   *   The array of extracted key/value pairs, or NULL if the string is invalid.
   *
   */
  protected static function extractAllowedValues($string) {
    $values = [];
    $list = explode("\n", $string);
    $list = array_map('trim', $list);
    $list = array_filter($list, 'strlen');
    $generated_keys = $explicit_keys = FALSE;
    foreach ($list as $position => $text) {
      $matches = [];
      if (preg_match('/(.+)+\|(.+)/', $text, $matches)) {
        // Trim key and value to avoid unwanted spaces issues.
        $key = trim($matches[1]);
        $value = trim($matches[2]);
        $explicit_keys = TRUE;
      }
      else {
        return;
      }
      $values[$key] = $value;
    }
    return $values;
  }


}
