<?php

/**
 * @file
 * Contains \Drupal\cforge\Installer\Setup2.
 */

namespace Drupal\cforge\Installer;

use Drupal\mcapi\Entity\Currency;
use Drupal\address\Entity\AddressFormat;
use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ExtensionDiscovery;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Choose between different Cforge flavours
 */
class Setup1 extends FormBase {

  private $installer;
  private $accountSwitcher;

  public function __construct($module_installer, $account_switcher) {
    $this->installer = $module_installer;
    $this->accountSwitcher = $account_switcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_installer'),
      $container->get('account_switcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_setup_1_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#title'] = $this->t('Setup stage 1');
    $country = strtoupper(\Drupal::config('system.date')->get('country.default'));
    $options = \Drupal::service('address.country_repository')->getList();

    if (isset($options[$country])) {
      $form['country'] = [
        '#type' => 'value',
        '#value' => $country
      ];
    }
    else {
      $form['country'] = [
        '#type' => 'select',
        '#title' => $this->t('Country'),
        '#options' => $options,
        '#required' => TRUE,
        '#size' => 12,
        '#weight' => 2
      ];
    }

    foreach(CFORGE_FLAVOURS as $mod => $name) {
      module_set_weight($mod, 100);
    }
    $form['flavour'] = [
      '#type' => $this->t('Flavour', [], ['context' => 'Installation variety']),
      '#type' => 'radios',
      '#options' => CFORGE_FLAVOURS,
      '#required' => TRUE,
      '#weight' => 4
    ];
    foreach ($form['flavour']['#options'] as $module => $name) {
      if ($message = cforge_flavour_install($module, TRUE)) {
        drupal_set_message($message, 'warning');
      }
    }

    $form['currency_type'] = [
      '#title' => $this->t('Main currency type'),
      '#type' => 'radios',
      '#options' => [
        'hours' => $this->t('Hours'),
        'other' => $this->t('Other'),
      ],
      '#weight' => 6,
      '#required' => TRUE
    ];
    $form['currency_name'] = [
      '#title' => $this->t('Currency name'),
      '#description' => $this->t('Use the plural'),
      '#type' => 'textfield',
      '#placeholder' => $this->t('Units'),
      '#weight' => 8,
      '#states' => [
        'visible' => [
          ':input[name="currency_type"]' => ['value' => 'other']
        ]
      ]
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'submit',
      '#weight' => 10
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    //finally set the addressfield to the country's addressFormat
    $country_code = $form_state->getValue('country');
    $field = FieldConfig::load('user.user.address');
    $settings = $field->get('settings');
    $settings['available_countries'][$country_code] = $country_code;
    $field->set('settings', $settings)->save();
    modify_address_format($country_code);

    //delete from existing config anything we find in the flavour's config directory
    $config_installer = \Drupal::service('config.installer');
    $flavour_module =  $form_state->getValue('flavour');

    $dir = drupal_get_path('module', $flavour_module) . '/' . \Drupal\Core\Config\InstallStorage::CONFIG_INSTALL_DIRECTORY;
    foreach (file_scan_directory($dir, '/.*yml/') as $path => $file) {
      if ($conf = \Drupal::configFactory()->getEditable($file->name)) {
        if ($conf->getRawData()) {
          $conf->delete();
          drupal_set_message('Deleted existing config object: '. $conf->getName());
        }
      }
    }
    $this->accountSwitcher->switchTo(\Drupal\user\Entity\User::load(1));
      if ($message = cforge_flavour_install($flavour_module, FALSE)) {
        drupal_set_message($message, 'warning');
      }
    $this->accountSwitcher->switchBack();


    //set up the currency by deleting the others
    if ($c = Currency::load('veur')) {
      $c->delete();
    }
    if ($form_state->getValue('currency_type') == 'hours') {
      if ($c = Currency::load('cc')) {
        $c->delete();
      }
    }
    else {
      if ($c = Currency::load('hhrs')) {
        $c->delete();
      }
      $curr = Currency::load('cc');
      $curr->name = $form_state->getValue('currency_name') ? : 'Units';
      $symbol = strtoupper(substr($curr->name, 0, 1));
      $curr->format = ["<strong>$symbol</strong>", '0000'];
      $curr->save();
    }

    //cforge_hosted if it is available
    $discovery = new ExtensionDiscovery(\Drupal::root());
    $available = $discovery->scan('module');
    if (isset($available['cforge_hosted'])) {
      //if cforge_hosted is installed, that means we are running under aegir
      $installer->install(['cforge_hosted']);
      drupal_set_message('Installed cforge_hosted module');
    }
    else {
      drupal_set_message('cforge_hosted module not present.');
    }


    global $install_state;
    $install_state['global'] = 1;
    $install_state['data_content'] = $form_state->getValue('data_content');

    $install_state = $form_state->getBuildInfo()['args'][0];
    $install_state['form_state'] = 1;
    $install_state['data_content'] = $form_state->getValue('data_content');
    $form_state->setBuildInfo(['args' => [$install_state]]);

    if ($install_state['data_content'] == 'example') {
      \Drupal::service('module_installer')->install(
        ['views_ui', 'field_ui', 'contextual', 'config', 'admin_toolbar_tools', 'devel', 'devel_generate']
      );
      \Drupal::configFactory()
        ->getEditable('views.settings')
        ->set('ui.show.sql_query.enabled', 1)
        ->set('ui.always_live_preview', 1)
        ->set('ui.show.advanced_column', 1)
        ->set('ui.show.master_display', 1)
        ->save();
    }
    elseif ($install_state['data_content'] == 'import') {
      \Drupal::service('module_installer')->install(['cforge_import']);
    }
  }

}

function modify_address_format($country_code) {
  $format = AddressFormat::load($country_code);
  if (!$format) {
    echo 'failed to load country ' . $country_code;
  }
  //if we have an altered version of the address, overwrite that country's address with ours
  if ($alt = AddressFormat::load('CF-' . $country_code)) {
    foreach ($alt->toArray() as $key => $value) {
      if ($key != 'countryCode' && $key != 'uuid') {
        $format->set($key, $value);
      }
    }
    $format->save();
  }
  //if we don't then bodge the existing format
  else {
    $pattern = $format->getFormat();
    $pattern .="\n%dependentLocality";
    $format->setFormat($pattern)
      ->setDependentLocalityType('neighborhood')
      ->setrequiredFields(['dependentLocality'])
      ->save();
  }

}
